"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HybridComplete = exports.ContextKind = void 0;
exports.computeStartCompletePosition = computeStartCompletePosition;
exports.analyzeCompletion = analyzeCompletion;
var language_1 = require("@codemirror/language");
var parser_1 = require("../parser");
var lezer_dql_1 = require("@cloudcare/lezer-dql");
var dql_terms_1 = require("./dql.terms");
var autocompleteNodes = {
    matchOp: dql_terms_1.matchOpTerms,
    connectOp: dql_terms_1.connectOpTerms,
    binOp: dql_terms_1.binOpTerms,
    duration: dql_terms_1.durationTerms,
    functionIdentifier: dql_terms_1.functionIdentifierTerms,
    number: [],
};
// ContextKind is the different possible value determinate by the autocompletion
var ContextKind;
(function (ContextKind) {
    // dynamic autocompletion (required a distant server)
    ContextKind[ContextKind["Namespace"] = 0] = "Namespace";
    ContextKind[ContextKind["Index"] = 1] = "Index";
    ContextKind[ContextKind["DataSource"] = 2] = "DataSource";
    ContextKind[ContextKind["ShowFunction"] = 3] = "ShowFunction";
    ContextKind[ContextKind["FieldName"] = 4] = "FieldName";
    ContextKind[ContextKind["FieldValue"] = 5] = "FieldValue";
    // static autocompletion
    ContextKind[ContextKind["Function"] = 6] = "Function";
    ContextKind[ContextKind["OuterFunction"] = 7] = "OuterFunction";
    ContextKind[ContextKind["BinOp"] = 8] = "BinOp";
    ContextKind[ContextKind["MatchOp"] = 9] = "MatchOp";
    ContextKind[ContextKind["ConnectOp"] = 10] = "ConnectOp";
    ContextKind[ContextKind["Duration"] = 11] = "Duration";
    ContextKind[ContextKind["Offset"] = 12] = "Offset";
    ContextKind[ContextKind["SOffset"] = 13] = "SOffset";
    ContextKind[ContextKind["Limit"] = 14] = "Limit";
    ContextKind[ContextKind["SLimit"] = 15] = "SLimit";
    ContextKind[ContextKind["Orderby"] = 16] = "Orderby";
    ContextKind[ContextKind["SOrderby"] = 17] = "SOrderby";
    ContextKind[ContextKind["By"] = 18] = "By";
    ContextKind[ContextKind["Number"] = 19] = "Number";
    ContextKind[ContextKind["Desc"] = 20] = "Desc";
    ContextKind[ContextKind["Asc"] = 21] = "Asc";
})(ContextKind || (exports.ContextKind = ContextKind = {}));
function trimQuotes(str) {
    // 1. 判断字符串首尾是否是同一个配对的引号
    if (!str)
        return str;
    // 支持 " ' `
    var first = str[0];
    var last = str[str.length - 1];
    if (first === last && "\"'`".includes(first)) {
        return str.slice(1, -1);
    }
    return str;
}
function computeStartCompleteWithQueted(node, pos) {
    // 有范围标志符号的 比如 {a,b,c}
    var start = node.from + 1;
    if (node.firstChild !== null) {
        start = pos;
    }
    return start;
}
function computeStartCompleteWithUnQueted(node, pos) {
    // 没有范围标志符号的 比如 a,b,c
    var start = node.from;
    if (node.firstChild !== null) {
        start = pos;
    }
    return start;
}
function computeStartCompletePosition(state, node, pos) {
    var _a, _b, _c, _d, _e, _f, _g;
    var start = node.from;
    if ((node.type.id === lezer_dql_1.Query && !(0, parser_1.containsChild)(node, lezer_dql_1.DataSourceList)) || node.type.id === lezer_dql_1.DColon) {
        // L:: and not container L::humber
        start = pos;
    }
    else if (node.type.id === lezer_dql_1.Number) {
        if (((_a = node.parent) === null || _a === void 0 ? void 0 : _a.type.id) === lezer_dql_1.TimeExpr) {
            start = pos;
        }
    }
    else if (node.type.id === lezer_dql_1.IndexExpr ||
        node.type.id === lezer_dql_1.InValue ||
        node.type.id === lezer_dql_1.Projection ||
        node.type.id === lezer_dql_1.FilterClause ||
        node.type.id === lezer_dql_1.InFieldValue) {
        // IndexExpr, InValue, Projection, FilterClause, InFieldValue
        // 这些节点的开始位置是从第一个子节点开始的
        start = computeStartCompleteWithQueted(node, pos);
    }
    else if (node.type.id === lezer_dql_1.DataSourceList || node.type.id === lezer_dql_1.FieldListExpr) {
        // DataSourceList
        // 这个节点的开始位置是从第一个子节点开始的
        start = computeStartCompleteWithUnQueted(node, pos);
    }
    else if ((node.type.id === lezer_dql_1.OuterFunctionCallBody || node.type.id === lezer_dql_1.EvalCallBody || node.type.id === lezer_dql_1.FunctionCallBody) &&
        node.firstChild === null) {
        start++;
    }
    else if (node.type.id === lezer_dql_1.StringLiteral &&
        ((0, parser_1.isMatcher)(node) ||
            ((_b = node.parent) === null || _b === void 0 ? void 0 : _b.type.id) === lezer_dql_1.IndexItem ||
            ((_c = node.parent) === null || _c === void 0 ? void 0 : _c.type.id) === lezer_dql_1.DataSourceItem ||
            ((_d = node.parent) === null || _d === void 0 ? void 0 : _d.type.id) === lezer_dql_1.InValue ||
            ((_e = node.parent) === null || _e === void 0 ? void 0 : _e.type.id) === lezer_dql_1.AsExpr ||
            ((_f = node.parent) === null || _f === void 0 ? void 0 : _f.type.id) === lezer_dql_1.FieldName ||
            ((_g = node.parent) === null || _g === void 0 ? void 0 : _g.type.id) === lezer_dql_1.InFieldValue)) {
        start++;
    }
    else if (node.type.id === lezer_dql_1.ByClause ||
        node.type.id === lezer_dql_1.OrderClause ||
        node.type.id === lezer_dql_1.SOrderClause ||
        node.type.id === lezer_dql_1.LimitClause ||
        node.type.id === lezer_dql_1.SLimitClause ||
        node.type.id === lezer_dql_1.OffsetClause ||
        node.type.id === lezer_dql_1.SOffsetClause) {
        start = pos;
    }
    var currentText = state.doc.slice(start, pos).toString();
    console.log('====currentText222', currentText);
    return start;
}
function arrayToCompletionResult(data, from, to, includeSnippet, span) {
    if (includeSnippet === void 0) { includeSnippet = false; }
    if (span === void 0) { span = true; }
    var options = data;
    if (includeSnippet) {
        options.push.apply(options, __spreadArray([], __read(dql_terms_1.snippets), false));
    }
    return {
        from: from,
        to: to,
        options: options,
        validFor: span ? /^[a-zA-Z0-9_:]+$/ : undefined,
    };
}
function getNodeDocByExpr(tree, state, finder) {
    var currentNode = tree.getChild(finder);
    if (!currentNode) {
        return '';
    }
    return state.sliceDoc(currentNode.from, currentNode.to);
}
function getNamespaceByQuery(tree, state) {
    var currentNode = (0, parser_1.walkBackward)(tree, lezer_dql_1.Query);
    if (!currentNode) {
        // Weird case that shouldn't happen, because "VectorSelector" is by definition the parent of the LabelMatchers.
        return '';
    }
    var namespace = '';
    currentNode.cursor().iterate(function (node) {
        // Continue until we find the VectorSelector, then look up the metric name.
        if (node.type.id === lezer_dql_1.Namespace) {
            namespace = getNodeDocByExpr(node.node, state, lezer_dql_1.Identifier);
            if (namespace) {
                return false;
            }
        }
    });
    return namespace;
}
function getIndexsByQuery(tree, state) {
    var currentNode = (0, parser_1.walkBackward)(tree, lezer_dql_1.Query);
    if (!currentNode) {
        // Weird case that shouldn't happen, because "VectorSelector" is by definition the parent of the LabelMatchers.
        return [];
    }
    var indexs = [];
    currentNode.cursor().iterate(function (node) {
        // Continue until we find the VectorSelector, then look up the metric name.
        if (node.type.id === lezer_dql_1.IndexExpr) {
            var IndexNodes = (0, parser_1.getChildNodeText)(node.node, lezer_dql_1.Identifier, lezer_dql_1.StringLiteral);
            IndexNodes.forEach(function (indexNode) {
                if (indexNode.type.id === lezer_dql_1.StringLiteral) {
                    indexs.push("".concat(state.sliceDoc(indexNode.from, indexNode.to).slice(1, -1)));
                }
                else if (indexNode.type.id === lezer_dql_1.Identifier) {
                    indexs.push(state.sliceDoc(indexNode.from, indexNode.to));
                }
            });
            return false;
        }
    });
    return indexs;
}
function getSourceListByQuery(tree, state) {
    var currentNode = (0, parser_1.walkBackward)(tree, lezer_dql_1.Query);
    if (!currentNode) {
        // Weird case that shouldn't happen, because "VectorSelector" is by definition the parent of the LabelMatchers.
        return [];
    }
    var sourcelist = [];
    currentNode.cursor().iterate(function (node) {
        // Continue until we find the VectorSelector, then look up the metric name.
        if (node.type.id === lezer_dql_1.DataSourceList) {
            var SourceNodes = (0, parser_1.getChildNodeText)(node.node, lezer_dql_1.Identifier, lezer_dql_1.StringLiteral);
            SourceNodes.forEach(function (sourceNode) {
                if (sourceNode.type.id === lezer_dql_1.StringLiteral) {
                    sourcelist.push("".concat(state.sliceDoc(sourceNode.from, sourceNode.to).slice(1, -1)));
                }
                else if (sourceNode.type.id === lezer_dql_1.Identifier) {
                    sourcelist.push(state.sliceDoc(sourceNode.from, sourceNode.to));
                }
            });
            return false;
        }
    });
    if (sourcelist.length === 0) {
        sourcelist = ['*'];
    }
    return sourcelist;
}
// analyzeCompletion is going to determinate what should be autocompleted.
// The value of the autocompletion is then calculate by the function buildCompletion.
// Note: this method is exported for testing purpose only. Do not use it directly.
function analyzeCompletion(state, node, pos) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q;
    var result = [];
    var currentText = state.doc.slice(node.from, pos).toString();
    console.log("currentText: ".concat(currentText), 'nodeName: ', node.type.name, getNamespaceByQuery(node, state), getIndexsByQuery(node, state), getSourceListByQuery(node, state));
    switch (node.type.id) {
        case 0: {
            // 0 is the id of the error node
        }
        case lezer_dql_1.DColon:
        case lezer_dql_1.Query: {
            result.push({ kind: ContextKind.DataSource, Namespace: getNamespaceByQuery(node, state), Index: getIndexsByQuery(node, state) });
            break;
        }
        case lezer_dql_1.DQL: {
            if (node.firstChild !== null && node.firstChild.type.id === 0) {
                // this situation can happen when there is nothing in the text area and the user is explicitly triggering the autocompletion (with ctrl + space)
                result.push({ kind: ContextKind.Namespace }, { kind: ContextKind.Function }, { kind: ContextKind.ShowFunction });
            }
            break;
        }
        case lezer_dql_1.IndexExpr: {
            result.push({ kind: ContextKind.Index, Namespace: getNamespaceByQuery(node, state) });
            break;
        }
        case lezer_dql_1.DataSourceList: {
            result.push({ kind: ContextKind.DataSource, Namespace: getNamespaceByQuery(node, state), Index: getIndexsByQuery(node, state) });
            break;
        }
        case lezer_dql_1.Projection: {
            result.push({
                kind: ContextKind.FieldName,
                Namespace: getNamespaceByQuery(node, state),
                Index: getIndexsByQuery(node, state),
                DataSource: getSourceListByQuery(node, state),
            });
            break;
        }
        case lezer_dql_1.FilterClause: {
            // 缺少 链接符的情况 {host = 'host' }
            //   if (node.firstChild) {
            //     result.push({
            //       kind: ContextKind.ConnectOp,
            //     });
            //   } else {
            //     result.push({
            //       kind: ContextKind.FieldName,
            //       Namespace: getNamespaceByQuery(node, state),
            //       Index: getIndexsByQuery(node, state),
            //       DataSource: getSourceListByQuery(node, state),
            //     });
            //   }
            break;
        }
        case lezer_dql_1.InFieldValue: {
            break;
        }
        case lezer_dql_1.TimeClause: {
            result.push({ kind: ContextKind.Number });
            break;
        }
        case lezer_dql_1.TimeRange: {
            result.push({ kind: ContextKind.Number });
            break;
        }
        case lezer_dql_1.FieldListExpr: {
            result.push({
                kind: ContextKind.FieldName,
                Namespace: getNamespaceByQuery(node, state),
                Index: getIndexsByQuery(node, state),
                DataSource: getSourceListByQuery(node, state),
            });
            break;
        }
        case lezer_dql_1.Identifier: {
            // Identifier is the root node of the DQL language
            // It can be used to complete the namespace, index, datasource, fieldname, fieldvalue
            if (((_a = node.parent) === null || _a === void 0 ? void 0 : _a.type.id) === lezer_dql_1.Namespace) {
                var queryNode = (0, parser_1.walkBackward)(node, lezer_dql_1.Query);
                if (queryNode === null || queryNode === void 0 ? void 0 : queryNode.getChildren(lezer_dql_1.DColon).length) {
                    //
                    result.push({
                        kind: ContextKind.Namespace,
                    });
                }
                else {
                    result.push({
                        kind: ContextKind.Namespace,
                    }, {
                        kind: ContextKind.ShowFunction,
                    }, {
                        kind: ContextKind.OuterFunction,
                    });
                }
            }
            else if (((_b = node.parent) === null || _b === void 0 ? void 0 : _b.type.id) === lezer_dql_1.FieldName && ((_d = (_c = node.parent) === null || _c === void 0 ? void 0 : _c.parent) === null || _d === void 0 ? void 0 : _d.type.id) === lezer_dql_1.ProjectionField) {
                // L::xxx:(avg(), host)
                result.push({
                    kind: ContextKind.FieldName,
                    Namespace: getNamespaceByQuery(node, state),
                    Index: getIndexsByQuery(node, state),
                    DataSource: getSourceListByQuery(node, state),
                }, { kind: ContextKind.Function });
            }
            else if (((_e = node.parent) === null || _e === void 0 ? void 0 : _e.type.id) === lezer_dql_1.ArgItem) {
                //Avg(xxx)
                result.push({
                    kind: ContextKind.FieldName,
                    Namespace: getNamespaceByQuery(node, state),
                    Index: getIndexsByQuery(node, state),
                    DataSource: getSourceListByQuery(node, state),
                });
            }
            else if (((_f = node.parent) === null || _f === void 0 ? void 0 : _f.type.id) === lezer_dql_1.IndexItem) {
                result.push({
                    kind: ContextKind.Index,
                    Namespace: getNamespaceByQuery(node, state),
                });
            }
            else if (((_g = node.parent) === null || _g === void 0 ? void 0 : _g.type.id) === lezer_dql_1.DataSourceItem) {
                result.push({
                    kind: ContextKind.DataSource,
                    Namespace: getNamespaceByQuery(node, state),
                    Index: getIndexsByQuery(node, state),
                });
            }
            else if (((_h = node.parent) === null || _h === void 0 ? void 0 : _h.type.id) === lezer_dql_1.FieldName) {
                result.push({
                    kind: ContextKind.FieldName,
                    Namespace: getNamespaceByQuery(node, state),
                    Index: getIndexsByQuery(node, state),
                    DataSource: getSourceListByQuery(node, state),
                });
            }
            else if (((_j = node.parent) === null || _j === void 0 ? void 0 : _j.type.id) === lezer_dql_1.FilterExpr) {
                result.push({
                    kind: ContextKind.FieldName,
                    Namespace: getNamespaceByQuery(node, state),
                    Index: getIndexsByQuery(node, state),
                    DataSource: getSourceListByQuery(node, state),
                });
            }
            break;
        }
        case lezer_dql_1.StringLiteral: {
            if (((_k = node.parent) === null || _k === void 0 ? void 0 : _k.type.id) === lezer_dql_1.IndexItem) {
                result.push({
                    kind: ContextKind.Index,
                    Namespace: getNamespaceByQuery(node, state),
                });
            }
            else if (((_l = node.parent) === null || _l === void 0 ? void 0 : _l.type.id) === lezer_dql_1.DataSourceItem) {
                result.push({
                    kind: ContextKind.DataSource,
                    Namespace: getNamespaceByQuery(node, state),
                    Index: getIndexsByQuery(node, state),
                });
            }
            else if (((_m = node.parent) === null || _m === void 0 ? void 0 : _m.type.id) === lezer_dql_1.InFieldValue) {
                if ((0, parser_1.isMatcher)(node)) {
                    result.push({
                        kind: ContextKind.FieldValue,
                        Namespace: getNamespaceByQuery(node, state),
                        Index: getIndexsByQuery(node, state),
                        DataSource: getSourceListByQuery(node, state),
                        matcher: (0, parser_1.createMatcher)(node, state),
                    });
                }
            }
            else if (((_o = node.parent) === null || _o === void 0 ? void 0 : _o.type.id) === lezer_dql_1.FieldName) {
                result.push({
                    kind: ContextKind.FieldName,
                    Namespace: getNamespaceByQuery(node, state),
                    Index: getIndexsByQuery(node, state),
                    DataSource: getSourceListByQuery(node, state),
                });
            }
            else if (((_p = node.parent) === null || _p === void 0 ? void 0 : _p.type.id) === lezer_dql_1.FilterExpr) {
                if ((0, parser_1.isMatcher)(node)) {
                    result.push({
                        kind: ContextKind.FieldValue,
                        Namespace: getNamespaceByQuery(node, state),
                        Index: getIndexsByQuery(node, state),
                        DataSource: getSourceListByQuery(node, state),
                        matcher: (0, parser_1.createMatcher)(node, state),
                    });
                }
                else {
                    result.push({
                        kind: ContextKind.FieldName,
                        Namespace: getNamespaceByQuery(node, state),
                        Index: getIndexsByQuery(node, state),
                        DataSource: getSourceListByQuery(node, state),
                    });
                }
            }
            break;
        }
        case lezer_dql_1.Number: {
            if (((_q = node.parent) === null || _q === void 0 ? void 0 : _q.type.id) === lezer_dql_1.TimeExpr) {
                result.push({
                    kind: ContextKind.Duration,
                });
            }
            else {
                result.push({
                    kind: ContextKind.Number,
                });
            }
            break;
        }
        case lezer_dql_1.ShowIdentifier: {
            result.push({
                kind: ContextKind.ShowFunction,
            });
            break;
        }
        case lezer_dql_1.EvalCallBody:
        case lezer_dql_1.FunctionCallBody: {
            break;
        }
        case lezer_dql_1.OuterFunctionCallBody: {
            //   result.push({
            //     kind: ContextKind.Function,
            //   });
            break;
        }
        case lezer_dql_1.InValue: {
            break;
        }
        case lezer_dql_1.EqlSingle:
        case lezer_dql_1.Neq:
        case lezer_dql_1.EqlRegex:
        case lezer_dql_1.NeqRegex:
        case lezer_dql_1.In:
        case lezer_dql_1.NotIn: {
            result.push({ kind: ContextKind.MatchOp });
            break;
        }
        case lezer_dql_1.By:
            break;
        case lezer_dql_1.OrderClause: {
            if (!node.getChildren(lezer_dql_1.By).length) {
                result.push({ kind: ContextKind.By });
            }
            else {
                result.push({
                    kind: ContextKind.FieldName,
                    Namespace: getNamespaceByQuery(node, state),
                    Index: getIndexsByQuery(node, state),
                    DataSource: getSourceListByQuery(node, state),
                });
            }
            break;
        }
        case lezer_dql_1.Order: {
            result.push({ kind: ContextKind.Orderby }, { kind: ContextKind.By });
            break;
        }
        case lezer_dql_1.SOrderClause: {
            if (!node.getChildren(lezer_dql_1.By).length) {
                result.push({ kind: ContextKind.By });
            }
            else {
                result.push({
                    kind: ContextKind.FieldName,
                    Namespace: getNamespaceByQuery(node, state),
                    Index: getIndexsByQuery(node, state),
                    DataSource: getSourceListByQuery(node, state),
                });
            }
            break;
        }
        case lezer_dql_1.SOrder:
            result.push({ kind: ContextKind.SOrderby }, { kind: ContextKind.By });
            break;
        case lezer_dql_1.Limit:
            result.push({ kind: ContextKind.Limit });
            break;
        case lezer_dql_1.Slimit:
            result.push({ kind: ContextKind.SLimit });
            break;
        case lezer_dql_1.Asc:
            result.push({ kind: ContextKind.Asc });
            break;
        case lezer_dql_1.Desc:
            result.push({ kind: ContextKind.Desc });
            break;
        case lezer_dql_1.And:
        case lezer_dql_1.DAnd:
        case lezer_dql_1.Or:
        case lezer_dql_1.DOr: {
            result.push({ kind: ContextKind.ConnectOp });
            break;
        }
        case lezer_dql_1.Pow:
        case lezer_dql_1.Mul:
        case lezer_dql_1.Div:
        case lezer_dql_1.Mod:
        case lezer_dql_1.Add:
        case lezer_dql_1.Sub:
        case lezer_dql_1.Eql:
        case lezer_dql_1.Gte:
        case lezer_dql_1.Gtr:
        case lezer_dql_1.Lte:
        case lezer_dql_1.Lss:
            result.push({ kind: ContextKind.BinOp });
            break;
        case lezer_dql_1.BinaryExpr:
            //       if ()
            //   result.push({ kind: ContextKind.BinOp });
            break;
    }
    return result;
}
var HybridComplete = /** @class */ (function () {
    function HybridComplete(dqlClient) {
        this.dqlClient = dqlClient;
    }
    HybridComplete.prototype.DQL = function (context) {
        var e_1, _a;
        var _this = this;
        var state = context.state, pos = context.pos;
        var tree = (0, language_1.syntaxTree)(state).resolve(pos, -1);
        // The lines above can help you to print the current lezer tree.
        // It's useful when you are trying to understand why it doesn't autocomplete.
        // console.log(syntaxTree(state).topNode.toString());
        // console.log(`current node: ${tree.type.name}`);
        var contexts = analyzeCompletion(state, tree, pos);
        var asyncResult = Promise.resolve([]);
        var completeSnippet = false;
        var span = true;
        var _loop_1 = function (context_1) {
            switch (context_1.kind) {
                case ContextKind.Namespace: {
                    asyncResult = asyncResult.then(function (result) {
                        return _this.autocompleteNamespace(result, context_1);
                    });
                    break;
                }
                case ContextKind.Index: {
                    asyncResult = asyncResult.then(function (result) {
                        return _this.autocompleteIndex(result, context_1);
                    });
                    break;
                }
                case ContextKind.DataSource: {
                    asyncResult = asyncResult.then(function (result) {
                        return _this.autocompleteDataSource(result, context_1);
                    });
                    break;
                }
                case ContextKind.ShowFunction: {
                    asyncResult = asyncResult.then(function (result) {
                        return _this.autocompleteShowFuncs(result, context_1);
                    });
                    break;
                }
                case ContextKind.FieldName: {
                    asyncResult = asyncResult.then(function (result) {
                        return _this.autocompleteFieldNames(result, context_1);
                    });
                    break;
                }
                case ContextKind.FieldValue: {
                    asyncResult = asyncResult.then(function (result) {
                        return _this.autocomplteFieldValues(result, context_1);
                    });
                    break;
                }
                case ContextKind.OuterFunction: {
                    asyncResult = asyncResult.then(function (result) {
                        return result.concat(autocompleteNodes.functionIdentifier);
                    });
                    break;
                }
                case ContextKind.Function: {
                    completeSnippet = true;
                    asyncResult = asyncResult.then(function (result) {
                        return result.concat(autocompleteNodes.functionIdentifier);
                    });
                    break;
                }
                case ContextKind.ConnectOp: {
                    asyncResult = asyncResult.then(function (result) {
                        return result.concat(autocompleteNodes.connectOp);
                    });
                    break;
                }
                case ContextKind.BinOp: {
                    asyncResult = asyncResult.then(function (result) {
                        return result.concat(autocompleteNodes.binOp);
                    });
                    break;
                }
                case ContextKind.MatchOp: {
                    asyncResult = asyncResult.then(function (result) {
                        return result.concat(autocompleteNodes.matchOp);
                    });
                    break;
                }
                case ContextKind.Duration: {
                    span = false;
                    asyncResult = asyncResult.then(function (result) {
                        return result.concat(autocompleteNodes.duration);
                    });
                    break;
                }
                case ContextKind.Offset: {
                    asyncResult = asyncResult.then(function (result) {
                        return result.concat([{ label: 'offset' }]);
                    });
                    break;
                }
                case ContextKind.SOffset: {
                    asyncResult = asyncResult.then(function (result) {
                        return result.concat([{ label: 'soffset' }]);
                    });
                    break;
                }
                case ContextKind.Limit: {
                    asyncResult = asyncResult.then(function (result) {
                        return result.concat([{ label: 'limit' }]);
                    });
                    break;
                }
                case ContextKind.SLimit: {
                    asyncResult = asyncResult.then(function (result) {
                        return result.concat([{ label: 'slimit' }]);
                    });
                    break;
                }
                case ContextKind.Orderby: {
                    asyncResult = asyncResult.then(function (result) {
                        return result.concat([{ label: 'order by' }]);
                    });
                    break;
                }
                case ContextKind.SOrderby: {
                    asyncResult = asyncResult.then(function (result) {
                        return result.concat([{ label: 'sorder by' }]);
                    });
                    break;
                }
                case ContextKind.By: {
                    asyncResult = asyncResult.then(function (result) {
                        return result.concat([{ label: 'by' }]);
                    });
                    break;
                }
                case ContextKind.Number: {
                    break;
                }
                case ContextKind.Asc: {
                    asyncResult = asyncResult.then(function (result) {
                        return result.concat([{ label: 'asc' }]);
                    });
                    break;
                }
                case ContextKind.Desc: {
                    asyncResult = asyncResult.then(function (result) {
                        return result.concat([{ label: 'desc' }]);
                    });
                    break;
                }
            }
        };
        try {
            for (var contexts_1 = __values(contexts), contexts_1_1 = contexts_1.next(); !contexts_1_1.done; contexts_1_1 = contexts_1.next()) {
                var context_1 = contexts_1_1.value;
                _loop_1(context_1);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (contexts_1_1 && !contexts_1_1.done && (_a = contexts_1.return)) _a.call(contexts_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return asyncResult.then(function (result) {
            return arrayToCompletionResult(result, computeStartCompletePosition(state, tree, pos), pos, completeSnippet, span);
        });
    };
    HybridComplete.prototype.autocompleteNamespace = function (result, context) {
        if (!this.dqlClient)
            return result;
        var namespaceCompetion = [];
        return this.dqlClient.namespace().then(function (namespaces) {
            namespaces.forEach(function (namespace) {
                namespaceCompetion.push({
                    label: namespace.name,
                    info: namespace.help,
                    type: 'constant',
                });
            });
            return namespaceCompetion;
        });
    };
    HybridComplete.prototype.autocompleteIndex = function (result, context) {
        if (!this.dqlClient)
            return result;
        var indexCompetion = [];
        return this.dqlClient.indexValues(context.Namespace).then(function (indexs) {
            indexs.forEach(function (index) {
                indexCompetion.push({
                    label: index,
                    type: 'keyword',
                });
            });
            return indexCompetion;
        });
    };
    HybridComplete.prototype.autocompleteDataSource = function (result, context) {
        if (!this.dqlClient)
            return result;
        var datasourceCompetion = [];
        return this.dqlClient.datasourceValues(context.Namespace, context.Index).then(function (datasources) {
            datasources.forEach(function (datasource) {
                datasourceCompetion.push({
                    label: datasource,
                    type: 'keyword',
                });
            });
            return datasourceCompetion;
        });
    };
    HybridComplete.prototype.autocompleteShowFuncs = function (result, context) {
        if (!this.dqlClient)
            return result;
        var showFuncCompetion = [];
        return this.dqlClient.showFunctions().then(function (showfuns) {
            showfuns.forEach(function (showfun) {
                showFuncCompetion.push({
                    label: showfun.name,
                    type: 'function',
                    detail: 'function',
                    info: showfun.info,
                });
            });
            return showFuncCompetion;
        });
    };
    HybridComplete.prototype.autocompleteFieldNames = function (result, context) {
        if (!this.dqlClient)
            return result;
        var fieldNamesCompetion = [];
        return this.dqlClient.fieldNames(context.Namespace, context.Index, context.DataSource).then(function (fieldNames) {
            fieldNames.forEach(function (fieldName) {
                fieldNamesCompetion.push({
                    label: fieldName.name,
                    detail: fieldName.type,
                    type: 'keyword',
                    info: fieldName.info,
                });
            });
            return fieldNamesCompetion;
        });
    };
    HybridComplete.prototype.autocomplteFieldValues = function (result, context) {
        if (!this.dqlClient)
            return result;
        var fieldValuesCompetion = [];
        return this.dqlClient.fieldValues(context.Namespace, context.Index, context.DataSource, context.matcher).then(function (fieldValues) {
            fieldValues.forEach(function (fieldValue) {
                fieldValuesCompetion.push({
                    label: fieldValue,
                    type: 'text',
                });
            });
            return fieldValuesCompetion;
        });
    };
    return HybridComplete;
}());
exports.HybridComplete = HybridComplete;
//# sourceMappingURL=hybrid.js.map