"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Parser = void 0;
var language_1 = require("@codemirror/language");
var Parser = /** @class */ (function () {
    function Parser(state) {
        this.tree = (0, language_1.syntaxTree)(state);
        this.state = state;
        this.diagnostics = [];
    }
    Parser.prototype.getDiagnostics = function () {
        return this.diagnostics.sort(function (a, b) {
            return a.from - b.from;
        });
    };
    Parser.prototype.analyze = function () {
        // when you are at the root of the tree, the first node is not `Expr` but a node with no name.
        // So to be able to iterate other the node relative to the promql node, we have to get the first child at the beginning
        this.checkAST(this.tree.topNode.firstChild);
        this.diagnoseAllErrorNodes();
    };
    Parser.prototype.diagnoseAllErrorNodes = function () {
        var cursor = this.tree.cursor();
        while (cursor.next()) {
            // usually there is an error node at the end of the expression when user is typing
            // so it's not really a useful information to say the expression is wrong.
            // Hopefully if there is an error node at the end of the tree, checkAST should yell more precisely
            if (cursor.type.id === 0 && cursor.to !== this.tree.topNode.to) {
                var node = cursor.node.parent;
                this.diagnostics.push({
                    severity: 'error',
                    message: 'unexpected expression',
                    from: node ? node.from : cursor.from,
                    to: node ? node.to : cursor.to,
                });
            }
        }
    };
    Parser.prototype.checkAST = function (node) { };
    Parser.prototype.addDiagnostic = function (node, msg) {
        this.diagnostics.push({
            severity: 'error',
            message: msg,
            from: node.from,
            to: node.to,
        });
    };
    return Parser;
}());
exports.Parser = Parser;
//# sourceMappingURL=parser.js.map