"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.walkBackward = walkBackward;
exports.getChildNodeText = getChildNodeText;
exports.containsAtLeastOneChild = containsAtLeastOneChild;
exports.containsChild = containsChild;
var lezer_dql_1 = require("@cloudcare/lezer-dql");
// walkBackward will iterate other the tree from the leaf to the root until it founds the given `exit` node.
// It returns null if the exit is not found.
function walkBackward(node, exit) {
    for (;;) {
        if (!node || node.type.id === exit) {
            return node;
        }
        node = node.parent;
    }
    return null;
}
function getChildNodeText(node) {
    var child = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        child[_i - 1] = arguments[_i];
    }
    var result = [];
    function dfsCursor(cursor) {
        // 如果遇到 Query 节点，停止递归
        if (cursor.type.name === 'Query' || cursor.type.id === lezer_dql_1.Query) {
            return;
        }
        if (child.some(function (t) { return cursor.type.id === t || cursor.type.name === t; })) {
            result.push(cursor.node);
        }
        if (cursor.firstChild()) {
            do {
                dfsCursor(cursor);
            } while (cursor.nextSibling());
            cursor.parent();
        }
    }
    var cursor = node.cursor();
    dfsCursor(cursor);
    return result;
}
function containsAtLeastOneChild(node) {
    var child = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        child[_i - 1] = arguments[_i];
    }
    var cursor = node.cursor();
    if (!cursor.next()) {
        // let's try to move directly to the children level and
        // return false immediately if the current node doesn't have any child
        return false;
    }
    var result = false;
    do {
        result = child.some(function (n) { return cursor.type.id === n || cursor.type.name === n; });
    } while (!result && cursor.nextSibling());
    return result;
}
function containsChild(node) {
    var child = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        child[_i - 1] = arguments[_i];
    }
    var cursor = node.cursor();
    if (!cursor.next()) {
        // let's try to move directly to the children level and
        // return false immediately if the current node doesn't have any child
        return false;
    }
    var i = 0;
    do {
        if (cursor.type.is(child[i])) {
            i++;
        }
    } while (i < child.length && cursor.nextSibling());
    return i >= child.length;
}
//# sourceMappingURL=path-finder.js.map