import { parser } from '@cloudcare/lezer-dql';
import { LRLanguage } from '@codemirror/language';
import { newCompleteStrategy } from './complete';
import { newLintStrategy, DQLLinter } from './lint';
export function DQLLanguage() {
    return LRLanguage.define({
        parser: parser,
        languageData: {
            closeBrackets: { brackets: ['(', '[', '{', "'", '"', '`'] },
            commentTokens: { line: '#' },
        },
    });
}
export class DQLExtension {
    constructor() {
        this.complete = newCompleteStrategy();
        this.lint = newLintStrategy();
        this.enableLinter = true;
        this.enableCompletion = true;
    }
    setComplete(dqlClent) {
        this.complete = newCompleteStrategy(dqlClent);
        return this;
    }
    getComplete() {
        return this.complete;
    }
    activateCompletion(activate) {
        this.enableCompletion = activate;
        return this;
    }
    setLinter(linter) {
        this.lint = linter;
        return this;
    }
    getLinter() {
        return this.lint;
    }
    activateLinter(activate) {
        this.enableLinter = activate;
        return this;
    }
    asExtension() {
        const language = DQLLanguage();
        let extension = [language];
        if (this.enableCompletion) {
            const completion = language.data.of({
                autocomplete: (context) => {
                    return this.complete.DQL(context);
                },
            });
            extension = extension.concat(completion);
        }
        if (this.enableLinter) {
            extension = extension.concat(DQLLinter(this.lint.promQL, this.lint));
        }
        return extension;
    }
}
//# sourceMappingURL=dql.js.map