import { syntaxTree } from '@codemirror/language';
export class Parser {
    constructor(state) {
        this.tree = syntaxTree(state);
        this.state = state;
        this.diagnostics = [];
    }
    getDiagnostics() {
        return this.diagnostics.sort((a, b) => {
            return a.from - b.from;
        });
    }
    analyze() {
        // when you are at the root of the tree, the first node is not `Expr` but a node with no name.
        // So to be able to iterate other the node relative to the promql node, we have to get the first child at the beginning
        this.checkAST(this.tree.topNode.firstChild);
        this.diagnoseAllErrorNodes();
    }
    diagnoseAllErrorNodes() {
        const cursor = this.tree.cursor();
        while (cursor.next()) {
            // usually there is an error node at the end of the expression when user is typing
            // so it's not really a useful information to say the expression is wrong.
            // Hopefully if there is an error node at the end of the tree, checkAST should yell more precisely
            if (cursor.type.id === 0 && cursor.to !== this.tree.topNode.to) {
                const node = cursor.node.parent;
                this.diagnostics.push({
                    severity: 'error',
                    message: 'unexpected expression',
                    from: node ? node.from : cursor.from,
                    to: node ? node.to : cursor.to,
                });
            }
        }
    }
    checkAST(node) { }
    addDiagnostic(node, msg) {
        this.diagnostics.push({
            severity: 'error',
            message: msg,
            from: node.from,
            to: node.to,
        });
    }
}
//# sourceMappingURL=parser.js.map