import { Query } from '@cloudcare/lezer-dql';
// walkBackward will iterate other the tree from the leaf to the root until it founds the given `exit` node.
// It returns null if the exit is not found.
export function walkBackward(node, exit) {
    for (;;) {
        if (!node || node.type.id === exit) {
            return node;
        }
        node = node.parent;
    }
    return null;
}
export function getChildNodeText(node, ...child) {
    const result = [];
    function dfsCursor(cursor) {
        // 如果遇到 Query 节点，停止递归
        if (cursor.type.name === 'Query' || cursor.type.id === Query) {
            return;
        }
        if (child.some((t) => cursor.type.id === t || cursor.type.name === t)) {
            result.push(cursor.node);
        }
        if (cursor.firstChild()) {
            do {
                dfsCursor(cursor);
            } while (cursor.nextSibling());
            cursor.parent();
        }
    }
    const cursor = node.cursor();
    dfsCursor(cursor);
    return result;
}
export function containsAtLeastOneChild(node, ...child) {
    const cursor = node.cursor();
    if (!cursor.next()) {
        // let's try to move directly to the children level and
        // return false immediately if the current node doesn't have any child
        return false;
    }
    let result = false;
    do {
        result = child.some((n) => cursor.type.id === n || cursor.type.name === n);
    } while (!result && cursor.nextSibling());
    return result;
}
export function containsChild(node, ...child) {
    const cursor = node.cursor();
    if (!cursor.next()) {
        // let's try to move directly to the children level and
        // return false immediately if the current node doesn't have any child
        return false;
    }
    let i = 0;
    do {
        if (cursor.type.is(child[i])) {
            i++;
        }
    } while (i < child.length && cursor.nextSibling());
    return i >= child.length;
}
//# sourceMappingURL=path-finder.js.map